@@ -109,6 +109,20 @@ class NetapiClient(object):
         local = salt.client.get_local_client(mopts=self.opts)
         return local.cmd_subset(*args, **kwargs)
 
+    def local_batch(self, *args, **kwargs):
+        '''
+        Run :ref:`execution modules <all-salt.modules>` against batches of minions
+
+        .. versionadded:: 0.8.4
+
+        Wraps :py:meth:`salt.client.LocalClient.cmd_batch`
+
+        :return: Returns the result from the exeuction module for each batch of
+            returns
+        '''
+        local = salt.client.get_local_client(mopts=self.opts)
+        return local.cmd_batch(*args, **kwargs)
+
     def ssh(self, *args, **kwargs):
         '''
         Run salt-ssh commands synchronously
