@@ -66,9 +66,9 @@ def orchestrate(mods,
     ret = {'data': {minion.opts['id']: running}, 'outputter': 'highstate'}
     res = salt.utils.check_state_result(ret['data'])
     if res:
-        ret['data']['retcode'] = 0
+        ret['retcode'] = 0
     else:
-        ret['data']['retcode'] = 1
+        ret['retcode'] = 1
     return ret
 
 # Aliases for orchestrate runner
