@@ -94,7 +94,14 @@ def mode(name):
     if tmode == 'unknown':
         ret['comment'] = '{0} is not an accepted mode'.format(name)
         return ret
+    # Either the current mode in memory or a non-matching config value
+    # will trigger setenforce
     mode = __salt__['selinux.getenforce']()
+    config = __salt__['selinux.getconfig']()
+    # Just making sure the oldmode reflects the thing that didn't match tmode
+    if mode == tmode and mode != config and tmode != config:
+        mode = config
+
     if mode == tmode:
         ret['result'] = True
         ret['comment'] = 'SELinux is already in {0} mode'.format(tmode)
@@ -109,7 +116,7 @@ def mode(name):
         return ret
 
     oldmode, mode = mode, __salt__['selinux.setenforce'](tmode)
-    if mode == tmode:
+    if mode == tmode or (tmode == 'Disabled' and __salt__['selinux.getconfig']() == tmode):
         ret['result'] = True
         ret['comment'] = 'SELinux has been set to {0} mode'.format(tmode)
         ret['changes'] = {'old': oldmode,
